/*
 *  Copyright (C) 2010  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/slab.h>

MODULE_LICENSE("GPL");

static dev_t devnum;
static struct cdev *dev;

static int hello_open(struct inode *ino, struct file *fp)
{
	printk(KERN_DEBUG "Hello, World!\n");
	return 0;
}

static const struct file_operations hello_fops = {
	.owner = THIS_MODULE,
	.open = hello_open,
};

static int __init ch_init(void)
{
	int r = 0;
	r = alloc_chrdev_region(&devnum, 0, 256, "hello");
	if (r)
		goto out;
	dev = cdev_alloc();
	if (!dev) {
		r = -ENOMEM;
		goto cdev_out;
	}
	dev->ops = &hello_fops;
	r = cdev_add(dev, devnum, 256);
	if (r)
		goto add_out;
	printk(KERN_DEBUG "Allocate major %d\n", MAJOR(devnum));
	return 0;
add_out:
	kfree(dev);
cdev_out:
	unregister_chrdev_region(devnum, 256);
out:
	return r;
}

static void __exit ch_exit(void)
{
	cdev_del(dev);
	unregister_chrdev_region(devnum, 256);
}

module_init(ch_init);
module_exit(ch_exit);
